-- HugeBagWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, Almagnus1, extended by Habna

-- HugeBag Known bugs:
-- When right clicking on a tool the icon is not swaping properly
-- Dragging the mouse beyond the edge of LotRO in Window-mode while doing a resize will cause HugeBag to get out of the screen
-- Unload command is not working
-- Reload command is not working

-- HugeBag future functions:
-- Sorting Capability
-- Add Setting Window

-- HugeBag change log:
-- 1.0.2 (2011/05/16) : Bug fix: Was unable to resize HugeBag when the 'reset' was launch.
--					  : Added a Watermark label of the authors Rod, Almagnus1 & Habna (me).
--					  : Added the ability to toggle the hide event when 'esc' key is press.
--					  : Change to Tobii source to be independant of Turbine plugins.
-- 1.0.1 (2011/05/15) : Added a context menu with right mouse click on the upper left corner.
-- 1.0.0 (2011/05/15) : Initial release.

import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

frmMain = class( Turbine.UI.Lotro.Window );

freeslots = 0; --> ** Global Variable to get the number of free slots **
backpackSize = self; --> ** Global variable to get the number of items in all bags **
InitXPos, InitYPos = 17,33; --> ** Initial X & Y position of the first Item in the HugeBag **
edgePadding = 15; --> ** HugeBag initial edgePadding value **
Version = "1.0.2"; --> ** HugeBag current version **
StartVisible, ShowSlotsInfo = true, true;
WinWidth, WinHeight = 460,300; --> ** HugeBag initial size **
WinLocX, WinLocY = self, self; --> ** HugeBag initial location **
settings, WMlabel = self, self;
HideWEsc = false; --> ** Initial value to determine if HugeBag hide when 'esc' key is press **
DefaultOpacity = 1; --> ** Initial opacity **

function frmMain:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	self:LoadSetting();
	self:SetVisible( StartVisible );

	--**v Helpfull section v**--
	-- **v To create a new window v**
	--self.window = Turbine.UI.Lotro.Window();
	--self.window:SetPosition( 100, 100 );
	--self.window:SetSize( 25, 25 );
	--self.window:SetText( "Title" );
	--self.window:SetVisible( true );
	-- **^
	-- **v To create a new button v**
	--self.button = Turbine.UI.Lotro.Button();
	--self.button:SetParent( self );
	--self.button:SetPosition( 5, 10 );
	--self.button:SetSize( 100, 150 );
	--self.button:SetText( "X" );
	--self.button:SetVisible( true );
	-- **^
	--**^

	--**v WaterMark Label v**
	WMlabel = Turbine.UI.Label();
	WMlabel:SetParent( self );
	WMlabel:SetText( "Originally written by Rod, Almagnus1, extended by Habna" );
	WMlabel:SetPosition( 20, WinHeight );
	WMlabel:SetSize(400, 25 );
	WMlabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	WMlabel:SetForeColor( Turbine.UI.Color( 1, 0.2, 0.2, 0.2 ) );
	WMlabel:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	--**^
	-- **v Setting to position the HugeBag at the bottom right of the screen v**
	self:SetSize( WinWidth, WinHeight );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( WinLocX, WinLocY );
	self:SetOpacity( DefaultOpacity );
	self:SetAllowDrop( true );
	self:SetWantsKeyEvents( true );
	-- **^	
	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	-- **^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );
	-- **^
	-- **v Set the itemlistbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	-- **^
	-- **v Setting up the contextMenuhandle v**
	self.contextMenuhandle = Turbine.UI.Control();
	contextMenuhandle = self.contextMenuhandle;
	contextMenuhandle:SetParent( self );
	--contextMenuhandle:SetZOrder( 100 );
	contextMenuhandle:SetSize( 20, 20 );
	contextMenuhandle:SetPosition( 5, 18 );

	-- **v Context menu settings v**
	self.menu = Turbine.UI.ContextMenu()
	self.menu.items = self.menu:GetItems()

	local option_line = Turbine.UI.MenuItem("-------------------------------------------------------------------", false, "line")

	local option_version = Turbine.UI.MenuItem("Show version of HugeBag in shell", true, "version")
	option_version.Click = VersionInfo

	local option_authors = Turbine.UI.MenuItem("Show authors of HugeBag in shell", true, "authors")
	option_authors.Click = AuthorsInfo

	local option_help = Turbine.UI.MenuItem("Show HugeBag commands", true, "help")
	option_help.Click = HelpInfo

	local option_reload = Turbine.UI.MenuItem("Reload HugeBag plugin instance", true, "reload")
	option_reload.Click = ReloadHugeBag

	local option_unload = Turbine.UI.MenuItem("Unload HugeBag plugin from memory", true, "unload")
	option_unload.Click = UnloadHugeBag

	local option_reset = Turbine.UI.MenuItem("Reset all settings to default", true, "reset")
	option_reset.Click = ResetSettings

	local option_hide = Turbine.UI.MenuItem("Hide HugeBag", true, "hide")
	option_hide.Click = HideHugeBag

	--local option_curset = Turbine.UI.MenuItem("Show the current settings", true, "curset")
	--option_curset.Click = ShowCurrentSettings

	local option_slotsinfo = Turbine.UI.MenuItem("Show free, used and max slots in shell", true, "slotsinfo")
	option_slotsinfo.Click = SlotsInfo

	local option_toggleslots = Turbine.UI.MenuItem("Toggle slots informations in title", true, "toggleslots")
	option_toggleslots.Click = ToggleSlotsInfo

	local option_togglewesc = Turbine.UI.MenuItem("Toggle hide event when the 'esc' key is press", true, "togglewesc")
	option_togglewesc.Click = ToggleWEsc

	local option_toggleshow = Turbine.UI.MenuItem("Toggle the startup visibility of HugeBag", true, "toggleshow")
	option_toggleshow.Click = ToggleShow

	self.menu.items:Add(option_slotsinfo)
	self.menu.items:Add(option_toggleshow)
	self.menu.items:Add(option_toggleslots)
	self.menu.items:Add(option_togglewesc)
	self.menu.items:Add(option_reset)
	self.menu.items:Add(option_line)
	self.menu.items:Add(option_version)
	self.menu.items:Add(option_authors)
	self.menu.items:Add(option_help)
	--self.menu.items:Add(option_curset)
	self.menu.items:Add(option_hide)
	self.menu.items:Add(option_line)
	self.menu.items:Add(option_reload)
	self.menu.items:Add(option_unload)

	-- contextMenuhandle event handlers
	self.contextMenuhandle.MouseDown = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.menu:ShowMenu()
		elseif args.Button == Turbine.UI.MouseButton.Right then
			
		end
	end
	-- **^
	-- **v Initial Setup for the resizehandle v**
	self.resizeHandle = Turbine.UI.Control();
	resizeHandle = self.resizeHandle;
	resizeHandle:SetParent( self );
	resizeHandle:SetZOrder( 100 );
	resizeHandle:SetSize( 25, 25 );
	resizeHandle:SetPosition( WinWidth - resizeHandle:GetWidth(), WinHeight - resizeHandle:GetHeight() );

	-- resizeHandle event handlers
	self.resizeHandle.MouseDown = function( sender, args )
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end

	self.resizeHandle.MouseUp = function( sender, args )
		sender.dragging = false;
		frmMain:SaveSetting()
	end

	self.resizeHandle.MouseMove = function( sender, args )
		if ( sender.dragging ) then
			WinWidth, WinHeight = self:GetSize();
			self:SetSize( WinWidth + ( args.X - sender.dragStartX ), WinHeight + ( args.Y - sender.dragStartY ) );
			sender:SetPosition( self:GetWidth() - sender:GetWidth(), self:GetHeight() - sender:GetHeight() )
			self:PerformLayout()
		end
	end
	-- **^
	
	self.items = { }; --> ** Initialize the container that holds the elements referencing the player's inventory **
	self:SetBackPack(); --> ** Set the backpack list **

	-- HugeBag event handlers
	-- **v Set the opacity when the mouse enter the HugeBag v**
	self.MouseEnter = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^
	-- **v Set the opacity when the mouse leave the HugeBag v**
	self.MouseLeave = function( sender, args )
		sender:SetOpacity( DefaultOpacity );
	end
	-- **^
	-- **v Do stuff before closing the window v**
	self.Closing = function( sender, args )
		
	end
	-- **^
	-- **v Refresh the X,Y positiov when HugeBag position change v**
	self.PositionChanged = function(args)
		WinLocX, WinLocY = self:GetPosition()
		frmMain:SaveSetting()
	end
	-- **^
	-- **v Detect key press v**
	self.KeyDown = function( sender, args )
		-- **v Hide HugeBag if ESC key is press v**
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			if ( HideWEsc ) then
				self:SetVisible( false );
			end
		end
		-- **^
		-- **v Show/Hide HugeBag if any bag was toggle v**
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
		-- **^
	end
	-- **^
	-- **v Adding an item v**
	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	-- **^
	-- **v Removing an item v**
	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	-- **^
	-- **v Changing place an item v**
	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();

		if ( shortcut ~= nil ) then
			local destinationItemControl = self.itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );

			-- if nil, then it's out of range, so don't do anything
			if ( destinationItemControl ~= nil ) then
				local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
				self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end
	-- **^
end
-- **^
-- **v Refresh the backpack list v**
function frmMain:SetBackPack()
	backpackSize = self.backpack:GetSize();
	local temp;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end

		-- Get item and set background color
		temp = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp:SetBackground( "HabnaPlugins/HugeBag/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		
		self.items[i] = temp;
		self.itemListBox:AddItem( self.items[i] );
	end
	self:PerformLayout();
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	WinWidth, WinHeight = self:GetSize();
	WinLocX, WinLocY = self:GetPosition()
	local itemWidth = 40;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = WinWidth - 40;
	local listHeight = WinHeight - 50;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( InitXPos, InitYPos );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	self.itemListBoxScrollBar:SetPosition( WinWidth - 17, 40 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );

	WMlabel:SetPosition( 30, WinHeight - 24 );

	self:CountFreeSlots()
end
-- **^
-- **v Count the free slots v**
function frmMain:CountFreeSlots()
	freeslots = 0;

	for i = 1, backpackSize, 1 do
		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end

	self:PrintTitle()
end
-- **^
-- **v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	if ( ShowSlotsInfo ) then
		self:SetText( "HugeBag - free slots: " .. freeslots .. " / " .. backpackSize );
	else
		self:SetText( "HugeBag" );
	end
end
-- **^
-- **v Show Help Info v**
function ReloadHugeBag()
	--Turbine.Shell.Execute( "/plugins load hugebag\r" );
	--Turbine.Shell.WriteLine( "HugeBag: loaded" );
	Turbine.Shell.WriteLine( "HugeBag: This function is currently not working" );
end
-- **^
-- **v Hide HugeBag v**
function HideHugeBag()
	if not frmMain:IsVisible() then
		Turbine.Shell.WriteLine( "HugeBag: Impossible, I'm already hidden" );
	else
		frmMain:SetVisible( false );
		--Turbine.Shell.WriteLine( "HugeBag: I'm now hidden" );
	end
end
-- **^
-- **v Hide HugeBag v**
function ShowHugeBag()
	if frmMain:IsVisible() then
		Turbine.Shell.WriteLine( "HugeBag: Impossible, I'm already visible" );
	else
		frmMain:SetVisible( true );
		--Turbine.Shell.WriteLine( "HugeBag: I'm now visible" );
	end
end
-- **^
-- **v Show Help Info v**
function UnloadHugeBag()
	--Turbine.Shell.Execute( "/plugins unload hugebag\r" );
	--Turbine.Shell.WriteLine( "HugeBag: unloaded" );
	Turbine.Shell.WriteLine( "HugeBag: This function is currently not working" );
end
-- **^
-- **v Show Version v**
function VersionInfo()
	Turbine.Shell.WriteLine( "HugeBag: My current version is: v" .. Version );
end
-- **^
-- **v Show Authors v**
function AuthorsInfo()
	Turbine.Shell.WriteLine( "HugeBag: My current authors are: Rod, Almagnus1, Extended by Habna" );
end
-- **^
-- **v Show Help Info v**
function HelpInfo()
	Turbine.Shell.WriteLine( "\n ** HugeBag v" .. Version .. " commands **\nshow: Show HugeBag\nhide: Hide HugeBag\nreset: Reset location, size and visibility to default\nslotsinfo: Show free, used and max slots in shell\ntogglewesc: Toggle hide event when the 'esc' key is press\nreload: Reload HugeBag plugin\nunload/doom: Unload HugeBag plugin from memory\ntoggleslots: Show or hide slots information in title\ntoggleshow: Show or hide HugeBag at startup\nhelp/?: Show HugeBag shell commands" );	
	--Turbine.Shell.WriteLine( "\n ** HugeBag v" .. Version .. " commands **\nshow: Show HugeBag\nhide: Hide HugeBag\nreset: Reset location, size and visibility to default\nslotsinfo: Show free, used and max slots in shell\ntogglewesc: Toggle hide event when the 'esc' key is press\nreload: Reload HugeBag plugin\nunload/doom: Unload HugeBag plugin from memory\ntoggleslots: Show or hide slots information in title\ntoggleshow: Show or hide HugeBag at startup\ncurset: Show HugeBag current settings\nhelp/?: Show HugeBag shell commands" );	
end
-- **^
-- **v Show Slots Info v**
function SlotsInfo()
	local usedslots = backpackSize - freeslots;
	Turbine.Shell.WriteLine( "HugeBag: I have " .. freeslots .. " free slots, " .. usedslots .. " used slots and " .. backpackSize .. " max slots" );
end
-- **^
-- **v Toggle Startup Visibility v**
function ToggleShow()
	StartVisible = ( not StartVisible );
	frmMain:SaveSetting()
	if ( StartVisible ) then
		Turbine.Shell.WriteLine( "HugeBag: I will load visibile next time" );
	else
		Turbine.Shell.WriteLine( "HugeBag: I will load hidden next time" );
	end
end
-- **^
-- **v Toggle Slots Info v**
function ToggleSlotsInfo()
	ShowSlotsInfo = ( not ShowSlotsInfo );
	frmMain:SaveSetting()
	frmMain:PrintTitle()
	if ( ShowSlotsInfo ) then
		Turbine.Shell.WriteLine( "HugeBag: Now showing slots informations in the title" );
	else
		Turbine.Shell.WriteLine( "HugeBag: Not showing slots informations in the title" );
	end
end
-- **^
-- **v Toggle the ability to hide are not HugeBag when the 'esc' key is press v**
function ToggleWEsc()
	HideWEsc = ( not HideWEsc );
	frmMain:SaveSetting()
	if ( HideWEsc ) then
		Turbine.Shell.WriteLine( "HugeBag: I will hide when 'esc' key is press" );
	else
		Turbine.Shell.WriteLine( "HugeBag: I will stay visible when 'esc' key is press" );
	end
end
-- **^
-- **v Load settings v**
function frmMain:LoadSetting()
	settings = {};
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "HugeBagSettings" );
	
	if settings == nil then
		self:SetSize( WinWidth, WinHeight );
		WinLocX = Turbine.UI.Display.GetWidth() - edgePadding - WinWidth
		WinLocY = Turbine.UI.Display.GetHeight() - edgePadding - WinHeight - 55
		self:SaveSetting()
	end

	--Version = settings.Version;
	StartVisible = settings.StartVisible;
	ShowSlotsInfo = settings.ShowSlotsInfo;
	WinLocX = settings.X;
	WinLocY = settings.Y;
	WinWidth = settings.Width;
	WinHeight = settings.Height;
	HideWEsc = settings.HideWEsc;
end
-- **^
-- **v Save settings v**
function frmMain:SaveSetting()
	settings = {};
	settings.Version = Version;
	settings.StartVisible = StartVisible;
	settings.ShowSlotsInfo = ShowSlotsInfo;
	settings.X = WinLocX; --Turbine.UI.Display.GetWidth() - edgePadding - self:GetWidth()
	settings.Y = WinLocY; --Turbine.UI.Display.GetHeight() - edgePadding - self:GetHeight() - 55
	settings.Width = self:GetWidth();
	settings.Height = self:GetHeight();
	settings.HideWEsc = HideWEsc;

	Turbine.PluginData.Save( Turbine.DataScope.Character, "HugeBagSettings", settings );
end
-- **^
-- **v Reset Settings v**
function ResetSettings()
	StartVisible = true;
	ShowSlotsInfo = true;
	HideWEsc = false;
	WinWidth = 460;
	WinHeight = 300;
	frmMain:SetSize( WinWidth, WinHeight );
	frmMain:SetPosition( Turbine.UI.Display.GetWidth() - edgePadding - WinWidth, Turbine.UI.Display.GetHeight() - edgePadding - WinHeight - 55 );
	resizeHandle:SetPosition( WinWidth - resizeHandle:GetWidth(), WinHeight - resizeHandle:GetHeight() );
	WinLocX, WinLocY = frmMain:GetPosition()
	frmMain:SaveSetting()
	frmMain:SetVisible( true );
	frmMain:PerformLayout()
	Turbine.Shell.WriteLine( "HugeBag: All my settings was set back to default" );
end
-- **^
-- **v Show Settings v**
function ShowCurrentSettings()
	
end
-- **^